@@ -39,6 +39,7 @@ from beneath.utils import (
   AIOWindowedBuffer,
   ms_to_datetime,
   ms_to_pd_timestamp,
+  StreamQualifier,
   timestamp_to_ms,
 )
 
@@ -55,20 +56,12 @@ class Stream:
   def __init__(
     self,
     client: Client,
-    project: str = None,
-    stream: str = None,
-    stream_id: str = None,
+    qualifier: StreamQualifier = None,
     max_write_delay_seconds: float = DEFAULT_WRITE_DELAY_SECONDS
   ):
-    # check stream_id xor (project, stream)
-    if bool(stream_id) == bool(project and stream):
-      raise ValueError("must provide either stream_id or (project and stream) parameters, but not both")
-
     # config
     self.client = client
-    self._project_name = project
-    self._stream_name = stream
-    self._stream_id = stream_id
+    self._qualifier = qualifier
 
     # prep state to load
     self.info: dict = None
@@ -86,18 +79,21 @@ class Stream:
 
   # LOADING STREAM CONTROL DATA
 
-  async def _ensure_loaded(self):
+  async def _ensure_loaded(self, prefetched=None, ensure_instance=False):
     if not self._loaded:
-      data = await self._load_from_admin()
-      self._set_admin_data(data)
+      if not prefetched:
+        prefetched = await self._load_from_admin()
+      self._set_admin_data(prefetched)
       self._loaded = True
+    if ensure_instance:
+      if not self.instance_id:
+        raise RuntimeError("Cannot query or write stream that doesn't have an instance")
 
   async def _load_from_admin(self):
-    if self._stream_id:
-      return await self.client.admin.streams.find_by_id(stream_id=self._stream_id)
-    return await self.client.admin.streams.find_by_project_and_name(
-      project_name=self._project_name,
-      stream_name=self._stream_name,
+    return await self.client.admin.streams.find_by_organization_project_and_name(
+      organization_name=self._qualifier.organization,
+      project_name=self._qualifier.project,
+      stream_name=self._qualifier.stream,
     )
 
   def _set_admin_data(self, data):
@@ -105,23 +101,21 @@ class Stream:
       "stream_id": data['streamID'],
       "stream_name": data['name'],
       "project_name": data['project']['name'],
+      "schema_kind": data['schemaKind'],
       "schema": data['schema'],
       "avro_schema": data['avroSchema'],
       "stream_indexes": data['streamIndexes'],
-      "external": data['external'],
-      "batch": data['batch'],
-      "manual": data['manual'],
       "retention_seconds": data['retentionSeconds'],
-      "current_instance_id": data['currentStreamInstanceID'],
+      "primary_instance_id": data['primaryStreamInstanceID'],
     }
-    self.instance_id = uuid.UUID(hex=self.info["current_instance_id"])
     self._avro_schema_parsed = parse_schema(json.loads(self.info['avro_schema']))
+    if self.info["primary_instance_id"]:
+      self.instance_id = uuid.UUID(hex=self.info["primary_instance_id"])
 
   # WRITING RECORDS
 
   async def write(self, records: Union[Iterable[Mapping], Mapping], immediate=False):
-    if not self._loaded:
-      await self._ensure_loaded()
+    await self._ensure_loaded(ensure_instance=True)
     if isinstance(records, Mapping):
       records = [records]
     batch = (self._record_to_pb(record) for record in records)
@@ -154,25 +148,22 @@ class Stream:
 
   # READING RECORDS
 
-  async def query(self, where: str = None) -> Cursor:
-    if not self._loaded:
-      await self._ensure_loaded()
-    resp = await self.client.connection.query(instance_id=self.instance_id, where=where)
+  async def query_log(self, peek: bool = False) -> Cursor:
+    await self._ensure_loaded(ensure_instance=True)
+    resp = await self.client.connection.query_log(instance_id=self.instance_id, peek=peek)
     assert len(resp.replay_cursors) <= 1 and len(resp.change_cursors) <= 1
     replay = resp.replay_cursors[0] if len(resp.replay_cursors) > 0 else None
     changes = resp.change_cursors[0] if len(resp.change_cursors) > 0 else None
     return Cursor(stream=self, instance_id=self.instance_id, replay_cursor=replay, changes_cursor=changes)
 
-  async def peek(self) -> Cursor:
-    if not self._loaded:
-      await self._ensure_loaded()
-    resp = await self.client.connection.peek(instance_id=self.instance_id)
-    return Cursor(
-      stream=self,
-      instance_id=self.instance_id,
-      replay_cursor=resp.rewind_cursor,
-      changes_cursor=resp.change_cursor,
-    )
+  # pylint: disable=redefined-builtin
+  async def query_index(self, filter: str = None) -> Cursor:
+    await self._ensure_loaded(ensure_instance=True)
+    resp = await self.client.connection.query_index(instance_id=self.instance_id, filter=filter)
+    assert len(resp.replay_cursors) <= 1 and len(resp.change_cursors) <= 1
+    replay = resp.replay_cursors[0] if len(resp.replay_cursors) > 0 else None
+    changes = resp.change_cursors[0] if len(resp.change_cursors) > 0 else None
+    return Cursor(stream=self, instance_id=self.instance_id, replay_cursor=replay, changes_cursor=changes)
 
   # MISC
 
